// Copyright 2020, Verizon Media Inc.

/**
 * @author Marian Rusnak <marian.rusnak@verizonmedia.com>
 */

import {ResponseStatus, SizeUnit} from '../data/enums';
import MobileMoments from './MobileMoments';
import objectUtils from '../utils/object';
import common from '../utils/common';

export default {
  /**
   * @param {string[]} positionNames
   * @param {JillResponse} response
   * @param {Config} config
   * @return {Object.<string, AdData>}
   */
  createFromJillResponse(positionNames, response, config) {
    const positions = response.positions;
    const l10nResource = response.client.l10n || {};
    return positionNames.reduce((adDataMap, positionName) => {
      const positionConfig = config.positions[positionName];
      // Only supports single ad response per position (no support for fetchMultiplier).
      const positionAdDataItems = positions[positionName];
      const adData = positionAdDataItems ? positions[positionName][0] : null;
      if (!positionConfig || !adData) {
        console.warn(`Position ${positionName} is not valid to be processed from the response.`);
        return adDataMap;
      }
      adData.config = objectUtils.clone(positionConfig);
      adData.config.client.closeButton.labels = {
        close: l10nResource.closeAdLabel,
        show: l10nResource.showAdLabel
      };
      const inputCloseBtn = positionConfig.input.client.closeButton || {};
      objectUtils.copy(positionConfig.client.closeButton.labels,
          adData.config.client.closeButton.labels, true);
      objectUtils.copy(inputCloseBtn.labels || {},
          adData.config.client.closeButton.labels);
      const configuredSizes = positionConfig.service.sizes;
      if (common.isString(configuredSizes)) {
        const sizeMapping = response.client.sizeMapping;
        if (common.isObject(sizeMapping) && sizeMapping[configuredSizes]) {
          adData.config.service.sizes = sizeMapping[configuredSizes];
        } else {
          console.warn(`Position ${positionName} size mapping lookup failure.`);
          return;
        }
      }
      adData.clientBaseUrl = response.client.url;
      adData.meta = adData.meta || {};
      adData.timestamp = new Date().getTime();
      // Used by publishers.
      adData.meta.isMobileMoment = MobileMoments.isMobileMomentAd(adData);

      adDataMap[positionName] = adData;
      return adDataMap;
    }, {});
  },

  /**
   * @param {string[]} positionNames
   * @param {Config} config
   * @return {Object.<string, AdData>}
   */
  createFromTimeout(positionNames, config) {
    return positionNames.reduce((adDataMap, positionName) => {
      const positionConfig = config.positions[positionName];
      if (!positionConfig) {
        console.warn(`Timeout for a non-existing position ${positionName} received.`);
        return adDataMap;
      }
      adDataMap[positionName] = {
        status: ResponseStatus.TIMEOUT,
        content: {},
        config: positionConfig,
        meta: {},
        timestamp: new Date().getTime()
      };
      return adDataMap;
    }, {});
  },

  /**
   * @param {PositionConfig} positionConfig
   * @return {AdData|null}
   */
  createFromContent(positionConfig) {
    const positionContent = positionConfig.client.content;
    if (positionContent && positionContent.markup) {
      positionConfig.service.sizes = positionConfig.service.sizes ||
        [`${positionContent.size.width}x${positionContent.size.height}`];
      return {
        status: ResponseStatus.OK,
        content: {
          markup: positionContent.markup,
          size: {
            ...positionContent.size,
            unit: SizeUnit.PIXEL
          }
        },
        config: positionConfig,
        meta: {},
        timestamp: new Date().getTime()
      };
    } else {
      return null;
    }
  }
};
